`timescale 1ns / 1ns
`include "constants.vams"

module cav_input_tb;

// quantities set in settings.vh
integer num, den, npt;
real harm, inrms, inamp;
reg signed [15:0] gain;  // < 32767/1.64676 = 19897
`include "settings.vh"

parameter dwell=4300;
reg clk;
integer cc;
initial begin
`ifdef CREATE_VCD_FILE
	$dumpfile("cav_input.vcd");
	$dumpvars(5,cav_input_tb);
`endif
	for (cc=0; cc<20+dwell+2*npt; cc=cc+1) begin
		clk = 0; #5;
		clk = 1; #5;
	end
	$finish;
end

// "computer controlled"
reg [16:0] phase_shift=0;

real volt, noise, poff=0;
integer nseed=1234;
reg signed [13:0] tmp=0, cavity_in=0;
always @(posedge clk) begin
	if (cc>dwell) poff=harm*(cc-dwell)/(2.0*npt);
	noise = $dist_normal(nseed,0,1024);
	volt = $sin(`M_TWO_PI*(poff+num*(cc%den)/(den*1.0)));
	if (cc>30) tmp = $floor(inamp*volt+noise*inrms/1024.0+0.5);
	// tmp = (cc%10==0)?4000:0;
	cavity_in <= tmp;
end

// The following section is normally done on the host computer;
// see rat_dds_config and rat_dds.v
reg [19:0] phase_step_h;
reg [11:0] phase_step_l, modulo;
integer m, r;
initial begin
	#1;  // wait for settings.vh to take effect
	m = 4096/den;
	modulo = 4096 - m*den;
	r = {1'b1,{20{1'b0}}}*num;
	phase_step_h =  r/den;
	phase_step_l = (r%den)*m;
	// $display("modulo = %d, phase_step = {%d,%d}", modulo, phase_step_h,phase_step_l);
end

// The following section is synthesizable and represents
// what would go in the final chip

// One DDS should control all input and output channels
wire [16:0] phase;
rat_dds dds(.clk(clk), .ang(phase),
	.phase_step_h(phase_step_h), .phase_step_l(phase_step_l),
	.modulo(modulo)
);

// controls interleave of I and Q
reg mod2 = 0;
always @(posedge clk) mod2 <= ~mod2;

wire signed [15:0] iq_out;
cav_input mut(.clk(clk),
	.cavity_in(cavity_in), .mod2(mod2), .phase(phase),
	.phase_shift(phase_shift), .gain(gain),
	.iq_out(iq_out)
);

// Print results suitable for checking with Octave
// Don't print while the CORDIC pipeline fills
reg signed [15:0] i_hold, q_hold;
always @(negedge clk) begin
	if (mod2) i_hold = iq_out;
	else      q_hold = iq_out;
	if (cc>=30) $display("%d %d %d %d %d", cavity_in, mut.cav_nodc, mod2, i_hold, q_hold);
end

endmodule
