// cordicg_tb.v
// Test bench for CORDIC routine, designed so post-processing can check accuracy
// $Id$
// Larry Doolittle, LBNL

`timescale 1ns / 1ns

module main();

reg clk, op;
initial clk=0;
integer cc;
reg signed [15:0] xin;
reg signed [15:0] yin=0;
reg signed [16:0] phasein=0;
initial begin
`ifdef CREATE_VCD_FILE
	$dumpfile("cordic.vcd");
	$dumpvars(5,main);
`endif
	if (!$value$plusargs("op=%d", op)) op=0;
	if (op) begin
		xin=19600;     // x^2 + y^2 needs to stay < 32767/1.64676 = 19897
	end else begin
		xin=13621;
	end
	for (cc=0; cc<4853; cc=cc+1) begin
		clk=0; #10;
		clk=1; #10;
	end
end

always @(posedge clk) if (op) begin
	yin <= yin + (xin>>>9) - (yin>>>14);
	xin <= xin - (yin>>>9) - (xin>>>14);
end else begin
	phasein<=phasein+27;
	yin<=yin+3;
end

wire signed [15:0] xout, yout;
wire [16:0] pout;
cordicg dut(clk, op, xin, yin, phasein, xout, yout, pout);
reg [16:0] ppout;
reg signed [15:0] xxout, yyout;
reg [16:0] pp[17:0];
reg signed [15:0] xp[17:0];
reg signed [15:0] yp[17:0];
reg [4:0] pipe_ix=0;
reg signed [16:0] p_show;
reg signed [15:0] x_show, y_show;
always @(posedge clk) begin
	xxout = xout;
	yyout = yout;
	ppout = pout;
	// Match the pipeline delay inside CORDIC.
	// You know when the lengths match when column 1 and 2 both become
	// non-x at the same time.
	
	pp[pipe_ix] <= phasein;
	xp[pipe_ix] <= xin;
	yp[pipe_ix] <= yin;
	p_show <= pp[pipe_ix];
	x_show <= xp[pipe_ix];
	y_show <= yp[pipe_ix];
	pipe_ix <= pipe_ix == 16 ? 0 : pipe_ix+1;
	$display("%6d %6d %6d %6d %6d %6d %d", p_show, x_show, y_show, xxout, yyout, ppout, op);
end
endmodule
