// Synthesizes to 80 4-LUTs and 2 MULT18X18s at 150 MHz in XC3Sxxx-4 using XST-10.1i
`timescale 1ns / 1ns

module downconvert(
	input clk,  // timespec 6.66 ns
	input  signed [14:0] wave_in,
	output signed [15:0] iq_out,
	input signed [17:0] cosd,
	input signed [17:0] sind,
	input mod2
);

// downconvert input signal to I and Q
// an extra pipeline stage has been added to help routing near multiplier
reg signed [31:0] mul_i=0, mul_q=0, mul_i1=0, mul_q1=0, mul_i2=0, mul_q2=0;
always @(posedge clk) begin
	mul_i  <= wave_in * cosd;
	mul_i1 <= mul_i;
	mul_i2 <= mul_i1;
	mul_q  <= wave_in * sind;
	mul_q1 <= mul_q;
	mul_q2 <= mul_q1;
end

wire signed [15:0] sum_i1, sum_q1;
sat_sub #(16, 16) si(clk, mul_i1[31:16], mul_i2[31:16], sum_i1);
sat_sub #(16, 16) sq(clk, mul_q2[31:16], mul_q1[31:16], sum_q1);

reg signed [15:0] sum_q2=0;
always @(posedge clk) sum_q2 <= sum_q1;
reg signed [15:0] iq_out0=0;
always @(posedge clk) iq_out0 <= mod2 ? sum_q2 : sum_i1;
assign iq_out = iq_out0;

endmodule
